/***************************************************************************//**
* \file main.c
* \version 1.0
*
* \brief
* Demonstrates blinking an LED under firmware control. The Cortex-M4 toggles
* the Red LED. The Cortex-M0+ starts the Cortex-M4 and enters sleep.
*
* Compatible Kits:
*   CY8CKIT-062-BLE
*   CY8CKIT-062-WIFI-BT
*
* Migration to CY8CPROTO-062-4343W kit (ModusToolbox IDE):
*   1. Create this project targeting the CY8CPROTO-062-4343W kit.
*   2. Open design.modus file and replicate P0[3] configuration on P13[7]. Give
*      this pin the alias "LED_RED". Disable P0[3].
*   3. Build and Program
* 
* Migration to CY8CPROTO-062-4343W kit (command line make):
*   1. Launch the Device Configurator tool from 
*      ModusToolbox_1.0\tools\device-configurator-1.0\
*   2. In Device Configurator, open design.modus file  
*      (ModusToolbox_1.0\libraries\psoc6sw-1.0\examples\BlinkyLED\design.modus) 
*      and replicate P0[3] configuration on P13[7]. 
*      Give this pin the alias "LED_RED". Disable P0[3].
*   3. Perform "make clean"
*   4. Build and Program the device with "make DEVICE=CY8C624ABZI-D44 program"
*      Note that depending on the method used to program the device, you may 
*      need to manually reset it by pressing the SW1 RESET button on the kit.
*   4. Observe the red blinking LED.
*   5. To switch back to CY8CKIT-062-BLE or CY8CKIT-062-WIFI-BT, 
*      perform steps 1 through 3 to reconfigure the "LED_RED" to P0[3]. 
*      Then use "make program".
*
********************************************************************************
* \copyright
* Copyright 2017-2019 Cypress Semiconductor Corporation
* SPDX-License-Identifier: Apache-2.0
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*     http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*******************************************************************************/
#include <stdio.h>
#include <string.h>

#include "cy_pdl.h"
#include "cycfg.h"
#include "RandomGen.h"

#include "stdlib.h"
#include "math.h"

uint32 i, k, j, k1;
uint32 Delay = {1};	//in milliseconds

/* Allocate context for SD Host operation */
cy_stc_sd_host_context_t sdHostContext;
/* And for UART */
cy_stc_scb_uart_context_t UART_context;

/*******************************************************************************
* Function Name: PrintArray
****************************************************************************//**
*
* This function prints the content of the RX buffer to the UART console.
*
* \param msg - message print before array output
*
* \param  rxBuffer - The buffer to the console output.
*
* \param  size - The size of the buffer to the console output.
*
*******************************************************************************/
void PrintArray(char * msg, uint8_t * buff, uint32_t size)
{
    printf("%s", msg);
    for(uint32_t index=0; index<size; index++)
    {
        printf("0x%02X ", (unsigned int) buff[index]);
    }
    printf("\r\n=======================\r\n");
    //printf("\r\nQuad is enabled. Enabling operation skipped\r\n");
}

/*******************************************************************************
* Function Name: Cy_SD_Host_SdCardChangeClock
****************************************************************************//**
*
*  Changes the Host controller SD clock.
*
* \param *base
*     The SD host registers structure pointer.
*
* \param frequency
*     The frequency in Hz.
*
* \return \ref cy_en_sd_host_status_t
*
*******************************************************************************/
static cy_en_sd_host_status_t Cy_SD_Host_SdCardChangeClock(SDHC_Type *base, uint32_t frequency)
{
    cy_en_sd_host_status_t ret = CY_SD_HOST_ERROR_INVALID_PARAMETER;
    uint32_t               clkDiv;
    uint32_t               clockInput = 100000000;//CY_SD_HOST_PERI_FREQUENCY;

    //CY_ASSERT_L2(CY_SD_HOST_IS_FREQ_VALID(frequency)); //not needed?

    if (NULL != base)
    {
        clkDiv = (clockInput / frequency) >> 1UL;
        Cy_SD_Host_DisableSdClk(base);
        ret = Cy_SD_Host_SetSdClkDiv(base, (uint16_t)clkDiv);
        Cy_SD_Host_EnableSdClk(base);
    }

    return ret;
}

/*******************************************************************************
* Function Name: Cy_SD_Host_ChangeIoVoltage
****************************************************************************//**
*
*  Changes the logic level on the sd_io_volt_sel line. It assumes that
*  this line is used to control a regulator connected to the VDDIO of the PSoC.
*  This regulator allows for switching between the 3.3V and 1.8V signaling.
*
* \note The dedicated io_volt_sel pin is used to change the regulator supplying
* voltage to the VDDIO of the SD block in order to operate at 1.8V. To configure
* the custom IO pin in order to control (using the GPIO driver) the regulator
* supplying voltage, the user must implement weak Cy_SD_Host_ChangeIoVoltage().
* Also, this function must set the SIGNALING_EN bit of the SDHC_CORE_HOST_CTRL2_R
* register when ioVoltage = CY_SD_HOST_IO_VOLT_1_8V.
*
* \param *base
*     The SD host registers structure pointer.
*
* \param ioVoltage
*     The voltage for IO.
*
*******************************************************************************/
__WEAK void Cy_SD_Host_ChangeIoVoltage(SDHC_Type *base, cy_en_sd_host_io_voltage_t ioVoltage)
{
	if (base == SDHC0)
			{
		if (CY_SD_HOST_IO_VOLT_1_8V == ioVoltage)  Cy_GPIO_Write(CARD0V18_PORT, CARD0V18_NUM, 1);
			                                 else  Cy_GPIO_Write(CARD0V18_PORT, CARD0V18_NUM, 0);
			}
	if (base == SDHC1)
			{
		if (CY_SD_HOST_IO_VOLT_1_8V == ioVoltage)  Cy_GPIO_Write(CARD1V18_PORT, CARD1V18_NUM, 1);
		                                 	 else  Cy_GPIO_Write(CARD1V18_PORT, CARD1V18_NUM, 0);
			}
	    /* Set the 1.8V signaling enable. */
	    SDHC_CORE_HOST_CTRL2_R(base) = _CLR_SET_FLD16U(SDHC_CORE_HOST_CTRL2_R(base),
	                                              SDHC_CORE_HOST_CTRL2_R_SIGNALING_EN,
	                                              (CY_SD_HOST_IO_VOLT_1_8V == ioVoltage) ? 1UL : 0UL);
}

/*******************************************************************************
* Function Name: Cy_SD_Host_IsCardConnected
****************************************************************************//**
*
*  Checks to see if a card is currently connected.
*
* \note You can use any GPIO custom pin for Card Detect. Add the SD Host driver
* Cy_SD_Host_IsCardConnected() function with the __WEAK type to your code.
* This function could read the value from any GPIO pin and return true when
* the card is connected.
*
* \param *base
*     The SD host registers structure pointer.
*
* \return bool
*     true - the card is connected, false - the card is removed (not connected).
*
*******************************************************************************/
__WEAK bool Cy_SD_Host_IsCardConnected(SDHC_Type const *base)
{
 if (base == SDHC1)	//only SDHC1 has card detect switch
    while(1UL == Cy_GPIO_Read(CARDDETECT_PORT, CARDDETECT_NUM))	//when no card high level
    {
        /* Wait until the card is stable. */
    }

 return true;	//Cy_GPIO_Read(CARDDETECT_PORT, CARDDETECT_NUM);
}

/*******************************************************************************
* Function Name: Cy_SD_Host_EnableCardVoltage
****************************************************************************//**
*
*  Sets the card_if_pwr_en pin high.
*  This pin can be used to enable a voltage regulator used to power the card.
*
* \param *base
*     The SD host registers structure pointer.
*
*******************************************************************************/
__WEAK void Cy_SD_Host_EnableCardVoltage(SDHC_Type *base)
{
    SDHC_CORE_PWR_CTRL_R(base) = _CLR_SET_FLD8U(SDHC_CORE_PWR_CTRL_R(base), SDHC_CORE_PWR_CTRL_R_SD_BUS_PWR_VDD1, 1UL);	//old code
    if (base == SDHC0)
    	Cy_GPIO_Write(CARD0VON_PORT, CARD0VON_NUM, 1);
    if (base == SDHC1)
        	Cy_GPIO_Write(CARD1VON_PORT, CARD1VON_NUM, 1);
}


/*******************************************************************************
* Function Name: Cy_SD_Host_DisableCardVoltage
****************************************************************************//**
*
*  Sets the card_if_pwr_en pin low.
*  This pin can be used to disable a voltage regulator used to power the card.
*
* \param *base
*     The SD host registers structure pointer.
*
*******************************************************************************/
__WEAK void Cy_SD_Host_DisableCardVoltage(SDHC_Type *base)
{

    SDHC_CORE_PWR_CTRL_R(base) = _CLR_SET_FLD8U(SDHC_CORE_PWR_CTRL_R(base), SDHC_CORE_PWR_CTRL_R_SD_BUS_PWR_VDD1, 0UL); //old code
    if (base == SDHC0)
    	Cy_GPIO_Write(CARD0VON_PORT, CARD0VON_NUM, 0);
    if (base == SDHC1)
    	Cy_GPIO_Write(CARD1VON_PORT, CARD1VON_NUM, 0);
}


int main(void)
{
	/* Set up internal routing, pins, and clock-to-peripheral connections */
    init_cycfg_all();
    
    Cy_SCB_UART_Init(UART_HW, &UART_config, &UART_context);
    Cy_SCB_UART_Enable(UART_HW);

    /* enable interrupts */
    __enable_irq();

    printf("\r\n\r\nUART initialization complete\r\n");
    printf("\r\n\r\nSDHC 100MHz code example started\r\n");
    printf("\r\nHere 1MB = 1,000,000 bytes\r\n");

    Cy_GPIO_Write(LED_RED_PORT, LED_RED_PIN, 0);	//Switch LED for 1 sec to indicate program start
    Cy_SysLib_Delay(1000/*msec*/);
    Cy_GPIO_Write(LED_RED_PORT, LED_RED_PIN, 1);

    //****************************************************************************************************************
    //Here we try to test new slot SDIO0, disabled for others because in his evaluation board this slot is absent
    //If card is not connected, the current code hangs.

    //Cy_SD_Host_Enable(SDHC0);

    //Cy_SD_Host_Init(SDHC0, &SDHC0_config, &sdHostContext);

    //Cy_SD_Host_InitCard(SDHC0, &SDHC0_card_cfg, &sdHostContext);

    //Cy_SD_Host_SetBusSpeedMode(SDHC0,CY_SD_HOST_BUS_SPEED_SDR50,&sdHostContext);	    //100MHz - works with 3V3 and 1.8V IOs
    //Cy_SD_Host_SdCardChangeClock(SDHC0, CY_SD_HOST_CLK_100M);						    //this is needed - works
    //Cy_SysLib_Delay(1); //msec, my addition, otherwise hangs

    /* Note: SD Host and the card must be initialized in ADMA2 or SDMA
     * mode before using the code below.
     */

    //The last is just to generate one random number to place it in the first sector
    uint32_t DATA, DATA1;	//Here will be random value, 8-bit
    InitRandomGen();
    DATA = GetRandomNumber();
    DATA1 = GetRandomNumber();

    printf("\r\nRandom values to put in the first sectors of two 384*1024-byte blocks: %d %d\r\n", DATA, DATA1);

    #define DATA_CLEAN   (0U)  /* To clean old buffer, value 0 */
    #define NSectors   (6*4*32UL)  //volume 384 Kbytes
    cy_stc_sd_host_write_read_config_t data;
    cy_en_sd_host_status_t ret;
    uint8_t txBuff[CY_SD_HOST_BLOCK_SIZE*NSectors];   /* Data to write. And also to read later*/

    for (i=0; i<NSectors; i++)	//Write incrementing in a loop 0-255 values, to each sector - one value
    	memset(txBuff+i*CY_SD_HOST_BLOCK_SIZE, DATA+i, CY_SD_HOST_BLOCK_SIZE);

    data.address = 0UL;         /* The address to write/read data on the card or eMMC. */
    data.numberOfBlocks = NSectors; //1UL;  /* The number of blocks to write/read (Single block write/read). */
    data.autoCommand = (1UL == (uint32_t)NSectors)
            ? CY_SD_HOST_AUTO_CMD_NONE
            : CY_SD_HOST_AUTO_CMD_AUTO;	//This is important for multi-sector operations
    data.dataTimeout = 12UL;     /* The timeout value for the transfer. */
    data.enReliableWrite = false; /* For EMMC cards enable reliable write. */
    data.enableDma = true;  /* Enable DMA mode. */
    data.data = (uint32_t*)txBuff;  /* The pointer to data to write and then to read. */

    //**********************************************************************************************************
    //Here is initialization for SDHC0 that is absent in the standard evaluation kit
    //ret = Cy_SD_Host_Write(SDHC0, &data, &sdHostContext);  /* Write data to the card. */
    //if (CY_SD_HOST_SUCCESS == ret)
    //{
    //    while (CY_SD_HOST_XFER_COMPLETE != (Cy_SD_Host_GetNormalInterruptStatus(SDHC0) & CY_SD_HOST_XFER_COMPLETE))
    //    {
    //        /* Wait for the data-transaction complete event. */
    //    }
    //}
    //data.data = (uint32_t*)rxBuff;  /* The pointer to data to read. */
    //ret = Cy_SD_Host_Read(SDHC0, &data, &sdHostContext);   /* Read data from the card. */
    //if (CY_SD_HOST_SUCCESS == ret)
    //{
    //    while (CY_SD_HOST_XFER_COMPLETE != (Cy_SD_Host_GetNormalInterruptStatus(SDHC0) & CY_SD_HOST_XFER_COMPLETE))
    //    {
    //        /* Wait for the data-transaction complete event. */
    //    }

    //    /* Clear the data-transaction complete event. */
    //    Cy_SD_Host_ClearNormalInterruptStatus(SDHC0, CY_SD_HOST_XFER_COMPLETE);
    //}

    //Cy_SysLib_Delay(1); //msec, my addition

    //k = 0;
    //for (i=0;i<CY_SD_HOST_BLOCK_SIZE;i++)
    //	if (txBuff[i] == rxBuff[i])	k++;

    //if (k==CY_SD_HOST_BLOCK_SIZE)
    //	for (i=0;i<10;i++)
    //    	{
    //    		Cy_GPIO_Inv(LED_RED_PORT, LED_RED_PIN); /* toggle the pin */
    //    		Cy_SysLib_Delay(100/*msec*/);
    //    	}
    //    else
    //  	for (i=0;i<2;i++)
    //   	    {
    //   	        Cy_GPIO_Inv(LED_RED_PORT, LED_RED_PIN); /* toggle the pin */
    //   	        Cy_SysLib_Delay(1000/*msec*/);
    //   	    }

    //**********************************************************************************************************
    //Here is the same but with SDIO1 (old slot)
    	Cy_SD_Host_Enable(SDHC1);

        Cy_SD_Host_Init(SDHC1, &SDHC1_config, &sdHostContext);

        Cy_SD_Host_InitCard(SDHC1, &SDHC1_card_cfg, &sdHostContext);

        //Cy_SD_Host_SetBusSpeedMode(SDHC1,CY_SD_HOST_BUS_SPEED_DEFAULT,&sdHostContext);	//25MHz - default, no need
        //Cy_SD_Host_SetBusSpeedMode(SDHC1,CY_SD_HOST_BUS_SPEED_HIGHSPEED,&sdHostContext);	//50MHz - works 3V3 and 1.8V IO
        //Cy_SD_Host_SetBusSpeedMode(SDHC1,CY_SD_HOST_BUS_SPEED_SDR25,&sdHostContext);	    //50MHz - works 3V3 and 1.8V IO
        //Cy_SD_Host_SdCardChangeClock(SDHC1, CY_SD_HOST_CLK_50M);						    //this is needed - works
        Cy_SD_Host_SetBusSpeedMode(SDHC1,CY_SD_HOST_BUS_SPEED_SDR50,&sdHostContext);	    //100MHz - works 3V3 and 1.8V IOt
        Cy_SD_Host_SdCardChangeClock(SDHC1, CY_SD_HOST_CLK_100M);						    //this is needed - works was 100M
        Cy_SysLib_Delay(Delay/*msec*/); //my addition, otherwise hangs

        /* Note: SD Host and the card must be initialized in ADMA2 or SDMA
         * mode before using the code below.
         */
        //#define DATA       (7U)  /* Data to write. value 7*/
        //#define DATA_OLD   (0U)  /* To clean old RX buffer, value 0 - my addition*/
        //cy_stc_sd_host_write_read_config_t data;
        //cy_en_sd_host_status_t ret;
        //uint8_t rxBuff[CY_SD_HOST_BLOCK_SIZE];   /* Data to read. */
        //uint8_t txBuff[CY_SD_HOST_BLOCK_SIZE];   /* Data to write. */
        //memset(txBuff, DATA, sizeof(txBuff));  /* Fill the array with data to write.  - Done before*/
        //memset(rxBuff, DATA_OLD, sizeof(rxBuff));  /* Fill the array with empty data for check - my addition */
        //data.address = 0UL;         /* The address to write/read data on the card or eMMC. */
        //data.numberOfBlocks = 1UL;  /* The number of blocks to write/read (Single block write/read). */
        //data.autoCommand = (1UL == (uint32_t)NSectors)
        //            ? CY_SD_HOST_AUTO_CMD_NONE
        //            : CY_SD_HOST_AUTO_CMD_AUTO;	//this helps!
        //data.dataTimeout = 12UL;     /* The timeout value for the transfer. */
        //data.enReliableWrite = false; /* For EMMC cards enable reliable write. */
        //data.enableDma = true;  /* Enable DMA mode. */
        //data.data = (uint32_t*)txBuff;  /* The pointer to data to write. */

        /* Scenario: there is need to initialize
         * the first (index = 0) counter of the TCPWM0 block
         * with the previously defined configuration settings
         */
        #define MY_TCPWM_CNT_NUM   (0UL)
        #define MY_TCPWM_CNT_MASK  (1UL << MY_TCPWM_CNT_NUM)

        if (CY_TCPWM_SUCCESS != Cy_TCPWM_Counter_Init(TCPWM0, MY_TCPWM_CNT_NUM, &TIMECOUNTER_config))
        {
            /* Handle possible errors */
        }
        /* Enable the initialized counter */
        Cy_TCPWM_Counter_Enable(TCPWM0, MY_TCPWM_CNT_NUM);
        Cy_TCPWM_Counter_SetCounter(TCPWM0, MY_TCPWM_CNT_NUM, 0);	//set 0, may be not needed
        Cy_TCPWM_TriggerStart(TCPWM0, MY_TCPWM_CNT_MASK);	//here we start to count

        Cy_GPIO_Write(CARD_BUSY_TEST_OUT_PORT, CARD_BUSY_TEST_OUT_NUM, 1);	//Indicate that SD writing stared
        ret = Cy_SD_Host_Write(SDHC1, &data, &sdHostContext);  /* Write data to the card. */
        if (CY_SD_HOST_SUCCESS == ret)
        {
            while (CY_SD_HOST_XFER_COMPLETE != (Cy_SD_Host_GetNormalInterruptStatus(SDHC1) & CY_SD_HOST_XFER_COMPLETE))
            {
                /* Wait for the data-transaction complete event. */
            }
        }
        Cy_GPIO_Write(CARD_BUSY_TEST_OUT_PORT, CARD_BUSY_TEST_OUT_NUM, 0);	//Indicate that SD writing finished
        //784 uS at 80 MHz bus; 150MHz core + 75MHz DMA is equal to 100MHz core + 100MHz DMA ->
        //some additional very essential delays are in the code.
        Cy_SysLib_Delay(Delay/*msec*/); //my addition

        Cy_TCPWM_TriggerStopOrKill(TCPWM0, MY_TCPWM_CNT_MASK);
        uint32_t WRITEDURATION = Cy_TCPWM_Counter_GetCounter(TCPWM0, MY_TCPWM_CNT_NUM);
        //printf("\r\n\r\Write duration of 384K in microseconds: %d\r\n",WRITEDURATION);
        //printf("\r\n\r\Write data rate:  %5.3f MBps (decimal)\r\n",(float)384*1024/WRITEDURATION);

        //write the second 384K block after the first one
        //Secondary INIT!!! without it does not work!
                Cy_SD_Host_InitCard(SDHC1, &SDHC1_card_cfg, &sdHostContext);
                Cy_SD_Host_SetBusSpeedMode(SDHC1,CY_SD_HOST_BUS_SPEED_SDR50,&sdHostContext);	    //100MHz - works 3V3 and 1.8V IOt
                Cy_SD_Host_SdCardChangeClock(SDHC1, CY_SD_HOST_CLK_100M);						    //this is needed - works was 100M
                Cy_SysLib_Delay(Delay/*msec*/); //my addition, otherwise hangs
        //END secondary INIT

        for (i=0; i<NSectors; i++)	//Write incrementing in a loop 0-255 values, to each sector - one value
            	memset(txBuff+i*CY_SD_HOST_BLOCK_SIZE, DATA1+i, CY_SD_HOST_BLOCK_SIZE);

        data.address = NSectors;

        Cy_TCPWM_Counter_SetCounter(TCPWM0, MY_TCPWM_CNT_NUM, 0);	//set 0, may be not needed
        Cy_TCPWM_TriggerStart(TCPWM0, MY_TCPWM_CNT_MASK);	//here we start to count
        Cy_GPIO_Write(CARD_BUSY_TEST_OUT_PORT, CARD_BUSY_TEST_OUT_NUM, 1);	//Indicate that SD writing stared
        ret = Cy_SD_Host_Write(SDHC1, &data, &sdHostContext);  /* Write data to the card. */
                if (CY_SD_HOST_SUCCESS == ret)
                {
                    while (CY_SD_HOST_XFER_COMPLETE != (Cy_SD_Host_GetNormalInterruptStatus(SDHC1) & CY_SD_HOST_XFER_COMPLETE))
                    {
                        /* Wait for the data-transaction complete event. */
                    }
                }
        Cy_GPIO_Write(CARD_BUSY_TEST_OUT_PORT, CARD_BUSY_TEST_OUT_NUM, 0);	//Indicate that SD writing finished
        Cy_TCPWM_TriggerStopOrKill(TCPWM0, MY_TCPWM_CNT_MASK);
        uint32_t WRITEDURATION1 = Cy_TCPWM_Counter_GetCounter(TCPWM0, MY_TCPWM_CNT_NUM);
        printf("\r\n\rWrite duration of 384*1024-byte blocks in microseconds: %d %d\r\n", WRITEDURATION, WRITEDURATION1);
        double MyMean, MyStd, V[2];
        V[0] = (double)384*1024/WRITEDURATION;
        V[1] = (double)384*1024/WRITEDURATION1;
        MyMean = (V[0]+V[1])/2;
        MyStd = fabs(V[0]-V[1])/sqrt(2);
        printf("\rWrite data rates:  %5.3f %5.3f MBps\r\n", V[0], V[1]);
        printf("\rMean+/-Std:  %5.3f+/-%5.3f MBps\r\n", MyMean, MyStd);

        //read from the original address
        data.address = 0UL;
        Cy_SysLib_Delay(Delay/*msec*/); //my addition, just to separate two intervals better

        //memset(txBuff, DATA_CLEAN, sizeof(txBuff));  /* Fill the array with zero data to clean it. */
        //Secondary INIT!!! without it does not work!
                        Cy_SD_Host_InitCard(SDHC1, &SDHC1_card_cfg, &sdHostContext);
                        Cy_SD_Host_SetBusSpeedMode(SDHC1,CY_SD_HOST_BUS_SPEED_SDR50,&sdHostContext);	    //100MHz - works 3V3 and 1.8V IOt
                        Cy_SD_Host_SdCardChangeClock(SDHC1, CY_SD_HOST_CLK_100M);						    //this is needed - works was 100M
                        Cy_SysLib_Delay(Delay/*msec*/); //my addition, otherwise hangs
                //END secondary INIT

        Cy_TCPWM_Counter_SetCounter(TCPWM0, MY_TCPWM_CNT_NUM, 0);	//set 0
        Cy_TCPWM_TriggerStart(TCPWM0, MY_TCPWM_CNT_MASK);	//here we start to count
        //data.data = (uint32_t*)txBuff;  /* The pointer to data to read. was rxBuff*/
        //data.numberOfBlocks = 1UL; //just to read one sector - temporarily!
        Cy_GPIO_Write(CARD_BUSY_TEST_OUT_PORT, CARD_BUSY_TEST_OUT_NUM, 1);	//Indicate that SD reading started
        ret = Cy_SD_Host_Read(SDHC1, &data, &sdHostContext);   /* Read data from the card. */
        if (CY_SD_HOST_SUCCESS == ret)
        {
            while (CY_SD_HOST_XFER_COMPLETE != (Cy_SD_Host_GetNormalInterruptStatus(SDHC1) & CY_SD_HOST_XFER_COMPLETE))
            {
                /* Wait for the data-transaction complete event. */
            }

            /* Clear the data-transaction complete event. */
            Cy_SD_Host_ClearNormalInterruptStatus(SDHC1, CY_SD_HOST_XFER_COMPLETE);
        }
        Cy_GPIO_Write(CARD_BUSY_TEST_OUT_PORT, CARD_BUSY_TEST_OUT_NUM, 0);	//Indicate that SD reading finished

        Cy_TCPWM_TriggerStopOrKill(TCPWM0, MY_TCPWM_CNT_MASK);
        uint32_t READDURATION = Cy_TCPWM_Counter_GetCounter(TCPWM0, MY_TCPWM_CNT_NUM);
        //printf("\r\n\r\Read duration of 384K in microseconds: %d\r\n",READDURATION);
        //printf("\r\n\r\Read data rate:  %5.3f MBps (decimal)\r\n",(float)384*1024/READDURATION);

        //Here we compare read values with the written value DATA
        uint8 DATA_t;
        uint32 i_t;
        bool Correct = true;
               for (i=0; i<NSectors; i++)	//Read incrementing in a loop 0-255 values, in each sector - one value
                   {
               	DATA_t = DATA+i;
               	i_t = i*CY_SD_HOST_BLOCK_SIZE;
                   for (j=0;j<CY_SD_HOST_BLOCK_SIZE;j++)
                  	   if (txBuff[i_t+j] != DATA_t)	Correct = false;
                   }

        data.address = NSectors;
        Cy_SysLib_Delay(Delay/*msec*/); //my addition
        //Secondary INIT!!! without it does not work!
                                Cy_SD_Host_InitCard(SDHC1, &SDHC1_card_cfg, &sdHostContext);
                                Cy_SD_Host_SetBusSpeedMode(SDHC1,CY_SD_HOST_BUS_SPEED_SDR50,&sdHostContext);	    //100MHz - works 3V3 and 1.8V IOt
                                Cy_SD_Host_SdCardChangeClock(SDHC1, CY_SD_HOST_CLK_100M);						    //this is needed - works was 100M
                                Cy_SysLib_Delay(Delay/*msec*/); //my addition, otherwise hangs
                        //END secondary INIT

                Cy_TCPWM_Counter_SetCounter(TCPWM0, MY_TCPWM_CNT_NUM, 0);	//set 0
                Cy_TCPWM_TriggerStart(TCPWM0, MY_TCPWM_CNT_MASK);	//here we start to count
                //data.data = (uint32_t*)txBuff;  /* The pointer to data to read. was rxBuff*/
                //data.numberOfBlocks = 1UL; //just to read one sector - temporarily!
                Cy_GPIO_Write(CARD_BUSY_TEST_OUT_PORT, CARD_BUSY_TEST_OUT_NUM, 1);	//Indicate that SD reading started
                ret = Cy_SD_Host_Read(SDHC1, &data, &sdHostContext);   /* Read data from the card. */
                if (CY_SD_HOST_SUCCESS == ret)
                {
                    while (CY_SD_HOST_XFER_COMPLETE != (Cy_SD_Host_GetNormalInterruptStatus(SDHC1) & CY_SD_HOST_XFER_COMPLETE))
                    {
                        /* Wait for the data-transaction complete event. */
                    }

                    /* Clear the data-transaction complete event. */
                    Cy_SD_Host_ClearNormalInterruptStatus(SDHC1, CY_SD_HOST_XFER_COMPLETE);
                }
                Cy_GPIO_Write(CARD_BUSY_TEST_OUT_PORT, CARD_BUSY_TEST_OUT_NUM, 0);	//Indicate that SD reading finished

                Cy_TCPWM_TriggerStopOrKill(TCPWM0, MY_TCPWM_CNT_MASK);
                uint32_t READDURATION1 = Cy_TCPWM_Counter_GetCounter(TCPWM0, MY_TCPWM_CNT_NUM);
                printf("\r\n\rRead duration of 384*1024-byte blocks in microseconds: %d %d\r\n", READDURATION, READDURATION1);
                       //float Mean, Std, V[2];
                       V[0] = (double)384*1024/READDURATION;
                       V[1] = (double)384*1024/READDURATION1;
                       MyMean = (V[0]+V[1])/2;
                       MyStd = fabs(V[0]-V[1])/sqrt(2);
                       printf("\rRead data rates:  %5.3f %5.3f MBps\r\n", V[0], V[1]);
                       printf("\rMean+/-Std:  %5.3f+/-%5.3f MBps\r\n", MyMean, MyStd);

        Cy_SysLib_Delay(1/*msec*/); //my addition, without if the following comparison gives wrong result

        //Here we compare read values with the written value DATA for the second block
        bool Correct1 = true;
                      for (i=0; i<NSectors; i++)	//Read incrementing in a loop 0-255 values, in each sector - one value
                          {
                      	DATA_t = DATA1+i;
                      	i_t = i*CY_SD_HOST_BLOCK_SIZE;
                          for (j=0;j<CY_SD_HOST_BLOCK_SIZE;j++)
                         	   if (txBuff[i_t+j] != DATA_t)	Correct1 = false;
                          }

        if (Correct && Correct1)	//If reading correct -> 5 short LED flashes
        	{
        	printf("\r\n\rSD Write/Read OK\r\n");
        	for (i=0; i<10; i++)
            	{
            		Cy_GPIO_Inv(LED_RED_PORT, LED_RED_PIN); /* toggle the pin */
            		Cy_SysLib_Delay(100/*msec*/);
            	}
        	}
            else				//If reading finished but wrong -> one 1-sec LED flash
           	    {
            		printf("\r\n\rSD Write/Read failed\r\n");
            		Cy_GPIO_Write(LED_RED_PORT, LED_RED_PIN, 0);	//Switch LED for 1 sec to indicate program start
            	    Cy_SysLib_Delay(1000/*msec*/);
            	    Cy_GPIO_Write(LED_RED_PORT, LED_RED_PIN, 1);
           	    }
}
